@@ -211,7 +211,7 @@ class Emitter:
         try:
             return self._stream
         except AttributeError:
-            raise YAMLStreamError('output stream needs to specified')
+            raise YAMLStreamError('output stream needs to be specified')
 
     @stream.setter
     def stream(self, val: Any) -> None:
@@ -748,7 +748,7 @@ class Emitter:
             and self.event.tag is not None
         ):
             if self.prepared_tag is None:
-                self.prepared_tag = self.prepare_tag(self.event.tag)
+                self.prepared_tag = self.prepare_tag(self.event.ctag)
             length += len(self.prepared_tag)
         if isinstance(self.event, ScalarEvent):
             if self.analysis is None:
@@ -813,7 +813,7 @@ class Emitter:
         if tag is None:
             raise EmitterError('tag is not specified')
         if self.prepared_tag is None:
-            self.prepared_tag = self.prepare_tag(tag)
+            self.prepared_tag = self.prepare_tag(self.event.ctag)
         if self.prepared_tag:
             self.write_indicator(self.prepared_tag, True)
             if (
@@ -825,6 +825,9 @@ class Emitter:
         self.prepared_tag = None
 
     def choose_scalar_style(self) -> Any:
+        # issue 449 needs this otherwise emits single quoted empty string
+        if self.event.value == '' and self.event.ctag.handle == '!!':
+            return None
         if self.analysis is None:
             self.analysis = self.analyze_scalar(self.event.value)
         if self.event.style == '"' or self.canonical:
@@ -956,6 +959,7 @@ class Emitter:
     def prepare_tag(self, tag: Any) -> Any:
         if not tag:
             raise EmitterError('tag must not be empty')
+        tag = str(tag)
         if tag == '!' or tag == '!!':
             return tag
         handle = None
@@ -1363,7 +1367,11 @@ class Emitter:
                     '\x20' <= ch <= '\x7E'
                     or (
                         self.allow_unicode
-                        and ('\xA0' <= ch <= '\uD7FF' or '\uE000' <= ch <= '\uFFFD')
+                        and (
+                            ('\xA0' <= ch <= '\uD7FF')
+                            or ('\uE000' <= ch <= '\uFFFD')
+                            or ('\U00010000' <= ch <= '\U0010FFFF')
+                        )
                     )
                 )
             ):
@@ -1378,11 +1386,11 @@ class Emitter:
                     if ch in self.ESCAPE_REPLACEMENTS:
                         data = '\\' + self.ESCAPE_REPLACEMENTS[ch]
                     elif ch <= '\xFF':
-                        data = f'\\x{ord(ch):02X}'
+                        data = '\\x%02X' % ord(ch)
                     elif ch <= '\uFFFF':
-                        data = f'\\u{ord(ch):04X}'
+                        data = '\\u%04X' % ord(ch)
                     else:
-                        data = f'\\U{ord(ch):08X}'
+                        data = '\\U%08X' % ord(ch)
                     self.column += len(data)
                     if bool(self.encoding):
                         data = data.encode(self.encoding)
@@ -1394,7 +1402,22 @@ class Emitter:
                 and self.column + (end - start) > self.best_width
                 and split
             ):
-                data = text[start:end] + '\\'
+                # SO https://stackoverflow.com/a/75634614/1307905
+                # data = text[start:end] + u'\\'  # <<< replaced with following six lines
+                need_backquote = True
+                if len(text) > end:
+                    try:
+                        space_pos = text.index(' ', end)
+                        if (
+                            '"' not in text[end:space_pos]
+                            and "'" not in text[end:space_pos]
+                            and text[space_pos + 1] != ' '
+                            and text[end - 1 : end + 1] != '  '
+                        ):
+                            need_backquote = False
+                    except (ValueError, IndexError):
+                        pass
+                data = text[start:end] + ('\\' if need_backquote else '')
                 if start < end:
                     start = end
                 self.column += len(data)
@@ -1405,7 +1428,11 @@ class Emitter:
                 self.whitespace = False
                 self.indention = False
                 if text[start] == ' ':
-                    data = '\\'
+                    if not need_backquote:
+                        # remove leading space it will load from the newline
+                        start += 1
+                    # data = u'\\'    # <<< replaced with following line
+                    data = '\\' if need_backquote else ''
                     self.column += len(data)
                     if bool(self.encoding):
                         data = data.encode(self.encoding)
@@ -1619,8 +1646,11 @@ class Emitter:
             else:
                 if ch is None or ch in ' \n\x85\u2028\u2029':
                     data = text[start:end]
-                    if len(data) > self.best_width and \
-                       self.column > self.indent:  # type: ignore
+                    if (
+                        len(data) > self.best_width
+                        and self.indent is not None
+                        and self.column > self.indent
+                    ):
                         # words longer than line length get a line of their own
                         self.write_indent()
                     self.column += len(data)
@@ -1697,3 +1727,26 @@ class Emitter:
         comment = event.comment[0]
         self.write_comment(comment)
         return True
+
+
+class RoundTripEmitter(Emitter):
+    def prepare_tag(self, ctag: Any) -> Any:
+        if not ctag:
+            raise EmitterError('tag must not be empty')
+        tag = str(ctag)
+        # print('handling', repr(tag))
+        if tag == '!' or tag == '!!':
+            return tag
+        handle = ctag.handle
+        suffix = ctag.suffix
+        prefixes = sorted(self.tag_prefixes.keys())
+        # print('handling', repr(tag), repr(suffix), repr(handle))
+        if handle is None:
+            for prefix in prefixes:
+                if tag.startswith(prefix) and (prefix == '!' or len(prefix) < len(tag)):
+                    handle = self.tag_prefixes[prefix]
+                    suffix = suffix[len(prefix) :]
+        if handle:
+            return f'{handle!s}{suffix!s}'
+        else:
+            return f'!<{suffix!s}>'
